@@ -272,6 +272,7 @@ module.exports = grammar({
       $.token_repetition,
       $.metavariable,
       $._non_special_token,
+      prec(-1, '$'),
     ),
 
     token_tree: $ => choice(
@@ -442,7 +443,7 @@ module.exports = grammar({
     ordered_field_declaration_list: $ => seq(
       '(',
       sepBy(',', seq(
-        optional($.attributes),
+        optional(field('attributes', $.attributes)),
         optional($.visibility_modifier),
         field('type', $._type),
       )),
@@ -651,7 +652,7 @@ module.exports = grammar({
     type_parameters: $ => prec(1, seq(
       '<',
       sepBy(',', seq(
-        optional($.attributes),
+        optional(field('attributes', $.attributes)),
         choice(
           $.metavariable,
           $.type_parameter,
@@ -764,7 +765,6 @@ module.exports = grammar({
     parameters: $ => seq(
       '(',
       sepBy(',', seq(
-        optional($.attributes),
         choice(
           $.parameter,
           $.self_parameter,
@@ -777,6 +777,7 @@ module.exports = grammar({
     ),
 
     self_parameter: $ => seq(
+      optional(field('attributes', $.attributes)),
       optional('&'),
       optional($.lifetime),
       optional($.mutable_specifier),
@@ -784,6 +785,7 @@ module.exports = grammar({
     ),
 
     variadic_parameter: $ => seq(
+      optional(field('attributes', $.attributes)),
       optional($.mutable_specifier),
       optional(seq(
         field('pattern', $._pattern),
@@ -793,6 +795,7 @@ module.exports = grammar({
     ),
 
     parameter: $ => seq(
+      optional(field('attributes', $.attributes)),
       optional($.mutable_specifier),
       field('pattern', choice(
         $._pattern,
